import importlib.metadata

from mhagenta.core import Orchestrator
from mhagenta.utils import State, Directory, Observation, Goal, Belief, ActionStatus
from mhagenta import bases, outboxes


__version__ = importlib.metadata.version("mhagenta")
__all__ = ['Orchestrator', 'State', 'Directory', 'Observation', 'Goal', 'Belief', 'ActionStatus', 'bases', 'outboxes', '__version__']



from abc import ABC, abstractmethod
from pydantic import BaseModel
import logging


DEFAULT_FORMAT = '[%(agent_time)f|%(mod_time)f|%(exec_time)s][%(levelname)s]::%(tags)s::%(message)s'


class LoggerExtras(BaseModel):
    agent_time: float
    mod_time: float
    exec_time: str
    tags: str


class ILogging(ABC):
    @property
    @abstractmethod
    def _logger_extras(self) -> LoggerExtras | None:
        pass

    @property
    @abstractmethod
    def _logger(self) -> logging.Logger:
        pass

    def log(self, level: int, message: str):
        self._logger.log(level, message, extra=dict() if self._logger_extras is None else self._logger_extras.model_dump())

    def debug(self, message: str):
        self._logger.debug(message, extra=dict() if self._logger_extras is None else self._logger_extras.model_dump())

    def info(self, message: str):
        self._logger.info(message, extra=dict() if self._logger_extras is None else self._logger_extras.model_dump())

    def warning(self, message: str):
        self._logger.warning(message, extra=dict() if self._logger_extras is None else self._logger_extras.model_dump())

    def error(self, message: str):
        self._logger.error(message, extra=dict() if self._logger_extras is None else self._logger_extras.model_dump())

    def critical(self, message: str):
        self._logger.critical(message, extra=dict() if self._logger_extras is None else self._logger_extras.model_dump())



from .process import MHAProcess
from .mha_module import GlobalParams, ModuleBase, MHAModule, run_agent_module
from .mha_root import MHARoot


__all__ = ['MHAProcess', 'GlobalParams', 'ModuleBase', 'run_agent_module', 'MHARoot', 'MHAModule']



import os
import dill
import asyncio
from typing import Any
import importlib.metadata

from mhagenta.core.processes import MHARoot
from mhagenta.modules import *
from mhagenta.states import *
from mhagenta.utils import ModuleTypes, Observation, ActionStatus
from mhagenta.core.processes import run_agent_module, MHAModule, ModuleBase, GlobalParams
from mhagenta.bases import *


async def main():
    with open('/agent/agent_params', 'rb') as f:
        params: dict[str, Any] = dill.load(f)

    id_override = os.environ.get('AGENT_ID')
    if id_override is not None and id_override != '':
        params['agent_id'] = os.environ.get('AGENT_ID')
    agent = MHARoot(**params)
    await agent.initialize()
    await agent.start()


if __name__ == '__main__':
    print(f'Using MHAgentA version {importlib.metadata.version("mhagenta")}')
    asyncio.run(main())



from .goals import GoalGraph, GoalGraphBase, GoalGraphOutbox, GoalGraphState
from .knowledge import Knowledge, KnowledgeBase, KnowledgeOutbox, KnowledgeState
from .high_level_reasoner import HLReasoner, HLReasonerBase, HLOutbox, HLState


__all__ = ['GoalGraph', 'GoalGraphBase', 'GoalGraphOutbox', 'GoalGraphState',
           'Knowledge', 'KnowledgeBase', 'KnowledgeOutbox', 'KnowledgeState',
           'HLReasoner', 'HLReasonerBase', 'HLOutbox', 'HLState']



from .processes import MHAProcess
from .orchestrator import Orchestrator
from .connection import Connector, RabbitMQConnector


__all__ = ['Connector', 'RabbitMQConnector', 'MHAProcess', 'Orchestrator']
